@@ -41,12 +41,14 @@ use uuid::Uuid;
 
 pub use self::bdaddr::{BDAddr, ParseBDAddrError};
 
+use crate::platform::PeripheralId;
+
 #[cfg_attr(
     feature = "serde",
     derive(Serialize, Deserialize),
     serde(crate = "serde_cr")
 )]
-#[derive(Debug, Clone, Eq, PartialEq)]
+#[derive(Debug, Clone, Copy, Eq, PartialEq)]
 pub enum AddressType {
     Random,
     Public,
@@ -112,6 +114,18 @@ impl Default for CharPropFlags {
     }
 }
 
+/// A GATT service. Services are groups of characteristics, which may be standard or
+/// device-specific.
+#[derive(Debug, Ord, PartialOrd, Eq, PartialEq, Clone)]
+pub struct Service {
+    /// The UUID for this service.
+    pub uuid: Uuid,
+    /// Whether this is a primary service.
+    pub primary: bool,
+    /// The characteristics of this service.
+    pub characteristics: BTreeSet<Characteristic>,
+}
+
 /// A Bluetooth characteristic. Characteristics are the main way you will interact with other
 /// bluetooth devices. Characteristics are identified by a UUID which may be standardized
 /// (like 0x2803, which identifies a characteristic for reading heart rate measurements) but more
@@ -124,6 +138,8 @@ impl Default for CharPropFlags {
 pub struct Characteristic {
     /// The UUID for this characteristic. This uniquely identifies its behavior.
     pub uuid: Uuid,
+    /// The UUID of the service this characteristic belongs to.
+    pub service_uuid: Uuid,
     /// The set of properties for this characteristic, which indicate what functionality it
     /// supports. If you attempt an operation that is not supported by the characteristics (for
     /// example setting notify on one without the NOTIFY flag), that operation will fail.
@@ -151,7 +167,9 @@ pub struct PeripheralProperties {
     /// The local name. This is generally a human-readable string that identifies the type of device.
     pub local_name: Option<String>,
     /// The transmission power level for the device
-    pub tx_power_level: Option<i8>,
+    pub tx_power_level: Option<i16>,
+    /// The most recent Received Signal Strength Indicator for the device
+    pub rssi: Option<i16>,
     /// Advertisement data specific to the device manufacturer. The keys of this map are
     /// 'manufacturer IDs', while the values are arbitrary data.
     pub manufacturer_data: HashMap<u16, Vec<u8>>,
@@ -160,8 +178,14 @@ pub struct PeripheralProperties {
     pub service_data: HashMap<Uuid, Vec<u8>>,
     /// Advertised services for this device
     pub services: Vec<Uuid>,
-    /// Number of times we've seen advertising reports for this device
-    pub discovery_count: u32,
+}
+
+/// The filter used when scanning for BLE devices.
+#[derive(Clone, Debug, Default, Eq, PartialEq)]
+pub struct ScanFilter {
+    /// If the filter contains at least one service UUID, only devices supporting at least one of
+    /// the given services will be available.
+    pub services: Vec<Uuid>,
 }
 
 /// The type of write operation to use.
@@ -179,6 +203,9 @@ pub enum WriteType {
 /// as well as functions for communication.
 #[async_trait]
 pub trait Peripheral: Send + Sync + Clone + Debug {
+    /// Returns the unique identifier of the peripheral.
+    fn id(&self) -> PeripheralId;
+
     /// Returns the MAC address of the peripheral.
     fn address(&self) -> BDAddr;
 
@@ -186,9 +213,18 @@ pub trait Peripheral: Send + Sync + Clone + Debug {
     /// as additional advertising reports are received.
     async fn properties(&self) -> Result<Option<PeripheralProperties>>;
 
+    /// The set of services we've discovered for this device. This will be empty until
+    /// `discover_services` is called.
+    fn services(&self) -> BTreeSet<Service>;
+
     /// The set of characteristics we've discovered for this device. This will be empty until
-    /// `discover_characteristics` is called.
-    fn characteristics(&self) -> BTreeSet<Characteristic>;
+    /// `discover_services` is called.
+    fn characteristics(&self) -> BTreeSet<Characteristic> {
+        self.services()
+            .iter()
+            .flat_map(|service| service.characteristics.clone().into_iter())
+            .collect()
+    }
 
     /// Returns true iff we are currently connected to the device.
     async fn is_connected(&self) -> Result<bool>;
@@ -201,8 +237,8 @@ pub trait Peripheral: Send + Sync + Clone + Debug {
     /// Terminates a connection to the device.
     async fn disconnect(&self) -> Result<()>;
 
-    /// Discovers all characteristics for the device.
-    async fn discover_characteristics(&self) -> Result<Vec<Characteristic>>;
+    /// Discovers all services for the device, including their characteristics.
+    async fn discover_services(&self) -> Result<()>;
 
     /// Write some data to the characteristic. Returns an error if the write couldn't be sent or (in
     /// the case of a write-with-response) if the device returns an error.
@@ -224,8 +260,9 @@ pub trait Peripheral: Send + Sync + Clone + Debug {
     async fn unsubscribe(&self, characteristic: &Characteristic) -> Result<()>;
 
     /// Returns a stream of notifications for characteristic value updates. The stream will receive
-    /// a notification when a value notification or indication is received from the device. This
-    /// method should only be used after a connection has been established.
+    /// a notification when a value notification or indication is received from the device.
+    /// The stream will remain valid across connections and can be queried before any connection
+    /// is made.
     async fn notifications(&self) -> Result<Pin<Box<dyn Stream<Item = ValueNotification> + Send>>>;
 }
 
@@ -236,24 +273,23 @@ pub trait Peripheral: Send + Sync + Clone + Debug {
 )]
 #[derive(Debug, Clone)]
 pub enum CentralEvent {
-    DeviceDiscovered(BDAddr),
-    DeviceLost(BDAddr),
-    DeviceUpdated(BDAddr),
-    DeviceConnected(BDAddr),
-    DeviceDisconnected(BDAddr),
+    DeviceDiscovered(PeripheralId),
+    DeviceUpdated(PeripheralId),
+    DeviceConnected(PeripheralId),
+    DeviceDisconnected(PeripheralId),
     /// Emitted when a Manufacturer Data advertisement has been received from a device
     ManufacturerDataAdvertisement {
-        address: BDAddr,
+        id: PeripheralId,
         manufacturer_data: HashMap<u16, Vec<u8>>,
     },
     /// Emitted when a Service Data advertisement has been received from a device
     ServiceDataAdvertisement {
-        address: BDAddr,
+        id: PeripheralId,
         service_data: HashMap<Uuid, Vec<u8>>,
     },
     /// Emitted when the advertised services for a device has been updated
     ServicesAdvertisement {
-        address: BDAddr,
+        id: PeripheralId,
         services: Vec<Uuid>,
     },
 }
@@ -271,7 +307,11 @@ pub trait Central: Send + Sync + Clone {
     /// Starts a scan for BLE devices. This scan will generally continue until explicitly stopped,
     /// although this may depend on your Bluetooth adapter. Discovered devices will be announced
     /// to subscribers of `events` and will be available via `peripherals()`.
-    async fn start_scan(&self) -> Result<()>;
+    /// The filter can be used to scan only for specific devices. While some implementations might
+    /// ignore (parts of) the filter and make additional devices available, other implementations
+    /// might require at least one filter for security reasons. Cross-platform code should provide
+    /// a filter, but must be able to handle devices, which do not fit into the filter.
+    async fn start_scan(&self, filter: ScanFilter) -> Result<()>;
 
     /// Stops scanning for BLE devices.
     async fn stop_scan(&self) -> Result<()>;
@@ -281,10 +321,16 @@ pub trait Central: Send + Sync + Clone {
     async fn peripherals(&self) -> Result<Vec<Self::Peripheral>>;
 
     /// Returns a particular [`Peripheral`] by its address if it has been discovered.
-    async fn peripheral(&self, address: BDAddr) -> Result<Self::Peripheral>;
+    async fn peripheral(&self, id: &PeripheralId) -> Result<Self::Peripheral>;
 
     /// Add a [`Peripheral`] from a MAC address without a scan result. Not supported on all Bluetooth systems.
     async fn add_peripheral(&self, address: BDAddr) -> Result<Self::Peripheral>;
+
+    /// Get information about the Bluetooth adapter being used, such as the model or type.
+    ///
+    /// The details of this are platform-specific andyou should not attempt to parse it, but it may
+    /// be useful for debug logs.
+    async fn adapter_info(&self) -> Result<String>;
 }
 
 /// The Manager is the entry point to the library, providing access to all the Bluetooth adapters on
