@@ -289,7 +289,16 @@ pub mod cb {
         unsafe { msg_send![cbperipheral, name] }
     }
 
-    pub fn peripheral_state(cbperipheral: *mut Object) -> c_int {
+    #[derive(Copy, Clone, Debug, Eq, PartialEq)]
+    #[repr(i64)]
+    pub enum CBPeripheralState {
+        Disonnected = 0,
+        Connecting = 1,
+        Connected = 2,
+        Disconnecting = 3,
+    }
+
+    pub fn peripheral_state(cbperipheral: *mut Object) -> CBPeripheralState {
         unsafe { msg_send![cbperipheral, state] }
     }
 
@@ -374,12 +383,9 @@ pub mod cb {
 
     // CBService : CBAttribute
 
-    // pub fn service_isprimary(cbservice: *mut Object) -> BOOL {
-    //     unsafe {
-    //         let isprimary: BOOL = msg_send![cbservice, isPrimary];
-    //         isprimary
-    //     }
-    // }
+    pub fn service_isprimary(cbservice: *mut Object) -> BOOL {
+        unsafe { msg_send![cbservice, isPrimary] }
+    }
 
     pub fn service_includedservices(cbservice: *mut Object) -> *mut Object /* NSArray<CBService*>* */
     {
@@ -405,6 +411,10 @@ pub mod cb {
         unsafe { msg_send![cbcharacteristic, properties] }
     }
 
+    pub fn characteristic_service(cbcharacteristic: *mut Object) -> *mut Object /* CBService* */ {
+        unsafe { msg_send![cbcharacteristic, service] }
+    }
+
     // CBCharacteristicProperties = NSUInteger from CBCharacteristic.h
 
     pub const CHARACTERISTICPROPERTY_BROADCAST: c_uint = 0x01; // CBCharacteristicPropertyBroadcast
