@@ -11,19 +11,19 @@
 use super::{
     central_delegate::{CentralDelegate, CentralDelegateEvent},
     framework::{
-        cb::{self, CBManagerAuthorization},
+        cb::{self, CBManagerAuthorization, CBPeripheralState},
         ns,
     },
     future::{BtlePlugFuture, BtlePlugFutureStateShared},
     utils::{core_bluetooth::cbuuid_to_uuid, nsstring::nsstring_to_string, nsuuid_to_uuid},
 };
-use crate::api::{CharPropFlags, Characteristic, WriteType};
+use crate::api::{CharPropFlags, Characteristic, ScanFilter, Service, WriteType};
 use crate::Error;
 use futures::channel::mpsc::{self, Receiver, Sender};
 use futures::select;
 use futures::sink::SinkExt;
 use futures::stream::{Fuse, StreamExt};
-use log::{error, info, trace, warn};
+use log::{error, trace, warn};
 use objc::{
     rc::StrongPtr,
     runtime::{Object, YES},
@@ -109,7 +109,8 @@ impl CBCharacteristic {
 #[derive(Clone, Debug)]
 pub enum CoreBluetoothReply {
     ReadResult(Vec<u8>),
-    Connected(BTreeSet<Characteristic>),
+    Connected(BTreeSet<Service>),
+    State(CBPeripheralState),
     Ok,
     Err(String),
 }
@@ -126,27 +127,32 @@ pub enum CBPeripheralEvent {
 pub type CoreBluetoothReplyStateShared = BtlePlugFutureStateShared<CoreBluetoothReply>;
 pub type CoreBluetoothReplyFuture = BtlePlugFuture<CoreBluetoothReply>;
 
+struct ServiceInternal {
+    cbservice: StrongPtr,
+    characteristics: HashMap<Uuid, CBCharacteristic>,
+}
+
 struct CBPeripheral {
     pub peripheral: StrongPtr,
-    services: HashMap<Uuid, StrongPtr>,
-    pub characteristics: HashMap<Uuid, CBCharacteristic>,
+    services: HashMap<Uuid, ServiceInternal>,
     pub event_sender: Sender<CBPeripheralEvent>,
     pub connected_future_state: Option<CoreBluetoothReplyStateShared>,
-    characteristic_update_count: u32,
 }
 
 impl Debug for CBPeripheral {
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
         f.debug_struct("CBPeripheral")
             .field("peripheral", self.peripheral.deref())
-            .field("services", &self.services.keys().collect::<Vec<_>>())
-            .field("characteristics", &self.characteristics)
+            .field(
+                "services",
+                &self
+                    .services
+                    .iter()
+                    .map(|(service_uuid, service)| (service_uuid, service.characteristics.len()))
+                    .collect::<HashMap<_, _>>(),
+            )
             .field("event_sender", &self.event_sender)
             .field("connected_future_state", &self.connected_future_state)
-            .field(
-                "characteristic_update_count",
-                &self.characteristic_update_count,
-            )
             .finish()
     }
 }
@@ -156,22 +162,28 @@ impl CBPeripheral {
         Self {
             peripheral,
             services: HashMap::new(),
-            characteristics: HashMap::new(),
             event_sender,
             connected_future_state: None,
-            characteristic_update_count: 0,
         }
     }
 
-    pub fn set_services(&mut self, services: HashMap<Uuid, StrongPtr>) {
-        self.services = services;
-    }
+    pub fn set_characteristics(
+        &mut self,
+        service_uuid: Uuid,
+        characteristics: HashMap<Uuid, StrongPtr>,
+    ) {
+        let characteristics = characteristics
+            .into_iter()
+            .map(|(characteristic_uuid, characteristic)| {
+                (characteristic_uuid, CBCharacteristic::new(characteristic))
+            })
+            .collect();
+        let service = self
+            .services
+            .get_mut(&service_uuid)
+            .expect("Got characteristics for a service we don't know about");
+        service.characteristics = characteristics;
 
-    pub fn set_characteristics(&mut self, characteristics: HashMap<Uuid, StrongPtr>) {
-        for (c_uuid, c_obj) in characteristics {
-            self.characteristics
-                .insert(c_uuid, CBCharacteristic::new(c_obj));
-        }
         // It's time for QUESTIONABLE ASSUMPTIONS.
         //
         // For sake of being lazy, we don't want to fire device connection until
@@ -180,26 +192,37 @@ impl CBPeripheral {
         // service map. Once that's done, we're filled out enough and can send
         // back a Connected reply to the waiting future with all of the
         // characteristic info in it.
-        self.characteristic_update_count += 1;
-        if self.characteristic_update_count == (self.services.len() as u32) {
+        if !self
+            .services
+            .values()
+            .any(|service| service.characteristics.is_empty())
+        {
             if self.connected_future_state.is_none() {
                 panic!("We should still have a future at this point!");
             }
-            let mut char_set = BTreeSet::new();
-            for (&uuid, c) in &self.characteristics {
-                let char = Characteristic {
-                    uuid,
-                    properties: c.properties,
-                };
-                trace!("{:?}", char.uuid);
-                char_set.insert(char);
-            }
+            let services = self
+                .services
+                .iter()
+                .map(|(&service_uuid, service)| Service {
+                    uuid: service_uuid,
+                    primary: cb::service_isprimary(*service.cbservice) != objc::runtime::NO,
+                    characteristics: service
+                        .characteristics
+                        .iter()
+                        .map(|(&characteristic_uuid, characteristic)| Characteristic {
+                            uuid: characteristic_uuid,
+                            service_uuid,
+                            properties: characteristic.properties,
+                        })
+                        .collect(),
+                })
+                .collect();
             self.connected_future_state
                 .take()
                 .unwrap()
                 .lock()
                 .unwrap()
-                .set_reply(CoreBluetoothReply::Connected(char_set));
+                .set_reply(CoreBluetoothReply::Connected(services));
         }
     }
 }
@@ -235,34 +258,65 @@ impl Debug for CoreBluetoothInternal {
 
 #[derive(Debug)]
 pub enum CoreBluetoothMessage {
-    StartScanning,
+    StartScanning {
+        filter: ScanFilter,
+    },
     StopScanning,
-    ConnectDevice(Uuid, CoreBluetoothReplyStateShared),
-    DisconnectDevice(Uuid, CoreBluetoothReplyStateShared),
-    // device uuid, characteristic uuid, future
-    ReadValue(Uuid, Uuid, CoreBluetoothReplyStateShared),
-    // device uuid, characteristic uuid, data, kind, future
-    WriteValue(
-        Uuid,
-        Uuid,
-        Vec<u8>,
-        WriteType,
-        CoreBluetoothReplyStateShared,
-    ),
-    // device uuid, characteristic uuid, future
-    Subscribe(Uuid, Uuid, CoreBluetoothReplyStateShared),
-    // device uuid, characteristic uuid, future
-    Unsubscribe(Uuid, Uuid, CoreBluetoothReplyStateShared),
+    ConnectDevice {
+        peripheral_uuid: Uuid,
+        future: CoreBluetoothReplyStateShared,
+    },
+    DisconnectDevice {
+        peripheral_uuid: Uuid,
+        future: CoreBluetoothReplyStateShared,
+    },
+    ReadValue {
+        peripheral_uuid: Uuid,
+        service_uuid: Uuid,
+        characteristic_uuid: Uuid,
+        future: CoreBluetoothReplyStateShared,
+    },
+    WriteValue {
+        peripheral_uuid: Uuid,
+        service_uuid: Uuid,
+        characteristic_uuid: Uuid,
+        data: Vec<u8>,
+        write_type: WriteType,
+        future: CoreBluetoothReplyStateShared,
+    },
+    Subscribe {
+        peripheral_uuid: Uuid,
+        service_uuid: Uuid,
+        characteristic_uuid: Uuid,
+        future: CoreBluetoothReplyStateShared,
+    },
+    Unsubscribe {
+        peripheral_uuid: Uuid,
+        service_uuid: Uuid,
+        characteristic_uuid: Uuid,
+        future: CoreBluetoothReplyStateShared,
+    },
+    IsConnected {
+        peripheral_uuid: Uuid,
+        future: CoreBluetoothReplyStateShared,
+    },
 }
 
 #[derive(Debug)]
 pub enum CoreBluetoothEvent {
     AdapterConnected,
-    // name, identifier, event receiver, message sender
-    DeviceDiscovered(Uuid, Option<String>, Receiver<CBPeripheralEvent>),
-    DeviceUpdated(Uuid, String),
-    // identifier
-    DeviceLost(Uuid),
+    DeviceDiscovered {
+        uuid: Uuid,
+        name: Option<String>,
+        event_receiver: Receiver<CBPeripheralEvent>,
+    },
+    DeviceUpdated {
+        uuid: Uuid,
+        name: String,
+    },
+    DeviceDisconnected {
+        uuid: Uuid,
+    },
 }
 
 impl CoreBluetoothInternal {
@@ -352,7 +406,7 @@ impl CoreBluetoothInternal {
         let name = nsstring_to_string(cb::peripheral_name(*peripheral));
         if self.peripherals.contains_key(&uuid) {
             if let Some(name) = name {
-                self.dispatch_event(CoreBluetoothEvent::DeviceUpdated(uuid, name))
+                self.dispatch_event(CoreBluetoothEvent::DeviceUpdated { uuid, name })
                     .await;
             }
         } else {
@@ -360,11 +414,11 @@ impl CoreBluetoothInternal {
             let (event_sender, event_receiver) = mpsc::channel(256);
             self.peripherals
                 .insert(uuid, CBPeripheral::new(peripheral, event_sender));
-            self.dispatch_event(CoreBluetoothEvent::DeviceDiscovered(
+            self.dispatch_event(CoreBluetoothEvent::DeviceDiscovered {
                 uuid,
                 name,
                 event_receiver,
-            ))
+            })
             .await;
         }
     }
@@ -379,21 +433,38 @@ impl CoreBluetoothInternal {
             trace!("{}", id);
         }
         if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
-            p.set_services(service_map);
+            let services = service_map
+                .into_iter()
+                .map(|(service_uuid, cbservice)| {
+                    (
+                        service_uuid,
+                        ServiceInternal {
+                            cbservice,
+                            characteristics: HashMap::new(),
+                        },
+                    )
+                })
+                .collect();
+            p.services = services;
         }
     }
 
     fn on_discovered_characteristics(
         &mut self,
         peripheral_uuid: Uuid,
-        char_map: HashMap<Uuid, StrongPtr>,
+        service_uuid: Uuid,
+        characteristics: HashMap<Uuid, StrongPtr>,
     ) {
-        trace!("Found chars!");
-        for id in char_map.keys() {
+        trace!(
+            "Found characteristics for peripheral {} service {}:",
+            peripheral_uuid,
+            service_uuid
+        );
+        for id in characteristics.keys() {
             trace!("{}", id);
         }
         if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
-            p.set_characteristics(char_map);
+            p.set_characteristics(service_uuid, characteristics);
         }
     }
 
@@ -402,74 +473,109 @@ impl CoreBluetoothInternal {
         // itself when it receives all of its service/characteristic info.
     }
 
-    async fn on_peripheral_disconnect(&mut self, peripheral_uuid: Uuid) {
-        self.peripherals.remove(&peripheral_uuid);
-        self.dispatch_event(CoreBluetoothEvent::DeviceLost(peripheral_uuid))
+    async fn on_peripheral_disconnect(&mut self, uuid: Uuid) {
+        trace!("Got disconnect event!");
+        self.peripherals.remove(&uuid);
+        self.dispatch_event(CoreBluetoothEvent::DeviceDisconnected { uuid })
             .await;
     }
 
-    fn on_characteristic_subscribed(&mut self, peripheral_uuid: Uuid, characteristic_uuid: Uuid) {
-        if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
-            if let Some(c) = p.characteristics.get_mut(&characteristic_uuid) {
-                trace!("Got subscribed event!");
-                let state = c.subscribe_future_state.pop_back().unwrap();
-                state.lock().unwrap().set_reply(CoreBluetoothReply::Ok);
-            }
+    /// Get the CBCharacteristic for the given characteristic of the given peripheral, if it exists.
+    fn get_characteristic(
+        &mut self,
+        peripheral_uuid: Uuid,
+        service_uuid: Uuid,
+        characteristic_uuid: Uuid,
+    ) -> Option<&mut CBCharacteristic> {
+        self.peripherals
+            .get_mut(&peripheral_uuid)?
+            .services
+            .get_mut(&service_uuid)?
+            .characteristics
+            .get_mut(&characteristic_uuid)
+    }
+
+    fn on_characteristic_subscribed(
+        &mut self,
+        peripheral_uuid: Uuid,
+        service_uuid: Uuid,
+        characteristic_uuid: Uuid,
+    ) {
+        if let Some(characteristic) =
+            self.get_characteristic(peripheral_uuid, service_uuid, characteristic_uuid)
+        {
+            trace!("Got subscribed event!");
+            let state = characteristic.subscribe_future_state.pop_back().unwrap();
+            state.lock().unwrap().set_reply(CoreBluetoothReply::Ok);
         }
     }
 
-    fn on_characteristic_unsubscribed(&mut self, peripheral_uuid: Uuid, characteristic_uuid: Uuid) {
-        if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
-            if let Some(c) = p.characteristics.get_mut(&characteristic_uuid) {
-                trace!("Got unsubscribed event!");
-                let state = c.unsubscribe_future_state.pop_back().unwrap();
-                state.lock().unwrap().set_reply(CoreBluetoothReply::Ok);
-            }
+    fn on_characteristic_unsubscribed(
+        &mut self,
+        peripheral_uuid: Uuid,
+        service_uuid: Uuid,
+        characteristic_uuid: Uuid,
+    ) {
+        if let Some(characteristic) =
+            self.get_characteristic(peripheral_uuid, service_uuid, characteristic_uuid)
+        {
+            trace!("Got unsubscribed event!");
+            let state = characteristic.unsubscribe_future_state.pop_back().unwrap();
+            state.lock().unwrap().set_reply(CoreBluetoothReply::Ok);
         }
     }
 
     async fn on_characteristic_read(
         &mut self,
         peripheral_uuid: Uuid,
+        service_uuid: Uuid,
         characteristic_uuid: Uuid,
         data: Vec<u8>,
     ) {
-        if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
-            if let Some(c) = p.characteristics.get_mut(&characteristic_uuid) {
-                trace!("Got read event!");
-
-                let mut data_clone = Vec::new();
-                for byte in data.iter() {
-                    data_clone.push(*byte);
-                }
-                // Reads and notifications both return the same callback. If
-                // we're trying to do a read, we'll have a future we can
-                // fulfill. Otherwise, just treat the returned value as a
-                // notification and use the event system.
-                if !c.read_future_state.is_empty() {
-                    let state = c.read_future_state.pop_back().unwrap();
-                    state
-                        .lock()
-                        .unwrap()
-                        .set_reply(CoreBluetoothReply::ReadResult(data_clone));
-                } else if let Err(e) = p
-                    .event_sender
-                    .send(CBPeripheralEvent::Notification(characteristic_uuid, data))
-                    .await
+        if let Some(peripheral) = self.peripherals.get_mut(&peripheral_uuid) {
+            if let Some(service) = peripheral.services.get_mut(&service_uuid) {
+                if let Some(characteristic) = service.characteristics.get_mut(&characteristic_uuid)
                 {
-                    error!("Error sending notification event: {}", e);
+                    trace!("Got read event!");
+
+                    let mut data_clone = Vec::new();
+                    for byte in data.iter() {
+                        data_clone.push(*byte);
+                    }
+                    // Reads and notifications both return the same callback. If
+                    // we're trying to do a read, we'll have a future we can
+                    // fulfill. Otherwise, just treat the returned value as a
+                    // notification and use the event system.
+                    if !characteristic.read_future_state.is_empty() {
+                        let state = characteristic.read_future_state.pop_back().unwrap();
+                        state
+                            .lock()
+                            .unwrap()
+                            .set_reply(CoreBluetoothReply::ReadResult(data_clone));
+                    } else if let Err(e) = peripheral
+                        .event_sender
+                        .send(CBPeripheralEvent::Notification(characteristic_uuid, data))
+                        .await
+                    {
+                        error!("Error sending notification event: {}", e);
+                    }
                 }
             }
         }
     }
 
-    fn on_characteristic_written(&mut self, peripheral_uuid: Uuid, characteristic_uuid: Uuid) {
-        if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
-            if let Some(c) = p.characteristics.get_mut(&characteristic_uuid) {
-                trace!("Got written event!");
-                let state = c.write_future_state.pop_back().unwrap();
-                state.lock().unwrap().set_reply(CoreBluetoothReply::Ok);
-            }
+    fn on_characteristic_written(
+        &mut self,
+        peripheral_uuid: Uuid,
+        service_uuid: Uuid,
+        characteristic_uuid: Uuid,
+    ) {
+        if let Some(characteristic) =
+            self.get_characteristic(peripheral_uuid, service_uuid, characteristic_uuid)
+        {
+            trace!("Got written event!");
+            let state = characteristic.write_future_state.pop_back().unwrap();
+            state.lock().unwrap().set_reply(CoreBluetoothReply::Ok);
         }
     }
 
@@ -482,32 +588,46 @@ impl CoreBluetoothInternal {
         }
     }
 
+    fn is_connected(&mut self, peripheral_uuid: Uuid, fut: CoreBluetoothReplyStateShared) {
+        if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
+            let state = cb::peripheral_state(*p.peripheral);
+            trace!("Connected state {:?} ", state);
+            fut.lock()
+                .unwrap()
+                .set_reply(CoreBluetoothReply::State(state));
+        }
+    }
+
     fn write_value(
         &mut self,
         peripheral_uuid: Uuid,
+        service_uuid: Uuid,
         characteristic_uuid: Uuid,
         data: Vec<u8>,
         kind: WriteType,
         fut: CoreBluetoothReplyStateShared,
     ) {
-        if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
-            if let Some(c) = p.characteristics.get_mut(&characteristic_uuid) {
-                trace!("Writing value! With kind {:?}", kind);
-                cb::peripheral_writevalue_forcharacteristic(
-                    *p.peripheral,
-                    ns::data(data.as_ptr(), data.len() as c_uint),
-                    *c.characteristic,
-                    match kind {
-                        WriteType::WithResponse => 0,
-                        WriteType::WithoutResponse => 1,
-                    },
-                );
-                // WriteWithoutResponse does not call the corebluetooth
-                // callback, it just always succeeds silently.
-                if kind == WriteType::WithoutResponse {
-                    fut.lock().unwrap().set_reply(CoreBluetoothReply::Ok);
-                } else {
-                    c.write_future_state.push_front(fut);
+        if let Some(peripheral) = self.peripherals.get_mut(&peripheral_uuid) {
+            if let Some(service) = peripheral.services.get_mut(&service_uuid) {
+                if let Some(characteristic) = service.characteristics.get_mut(&characteristic_uuid)
+                {
+                    trace!("Writing value! With kind {:?}", kind);
+                    cb::peripheral_writevalue_forcharacteristic(
+                        *peripheral.peripheral,
+                        ns::data(data.as_ptr(), data.len() as c_uint),
+                        *characteristic.characteristic,
+                        match kind {
+                            WriteType::WithResponse => 0,
+                            WriteType::WithoutResponse => 1,
+                        },
+                    );
+                    // WriteWithoutResponse does not call the corebluetooth
+                    // callback, it just always succeeds silently.
+                    if kind == WriteType::WithoutResponse {
+                        fut.lock().unwrap().set_reply(CoreBluetoothReply::Ok);
+                    } else {
+                        characteristic.write_future_state.push_front(fut);
+                    }
                 }
             }
         }
@@ -516,14 +636,21 @@ impl CoreBluetoothInternal {
     fn read_value(
         &mut self,
         peripheral_uuid: Uuid,
+        service_uuid: Uuid,
         characteristic_uuid: Uuid,
         fut: CoreBluetoothReplyStateShared,
     ) {
-        if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
-            if let Some(c) = p.characteristics.get_mut(&characteristic_uuid) {
-                trace!("Reading value!");
-                cb::peripheral_readvalue_forcharacteristic(*p.peripheral, *c.characteristic);
-                c.read_future_state.push_front(fut);
+        if let Some(peripheral) = self.peripherals.get_mut(&peripheral_uuid) {
+            if let Some(service) = peripheral.services.get_mut(&service_uuid) {
+                if let Some(characteristic) = service.characteristics.get_mut(&characteristic_uuid)
+                {
+                    trace!("Reading value!");
+                    cb::peripheral_readvalue_forcharacteristic(
+                        *peripheral.peripheral,
+                        *characteristic.characteristic,
+                    );
+                    characteristic.read_future_state.push_front(fut);
+                }
             }
         }
     }
@@ -531,18 +658,22 @@ impl CoreBluetoothInternal {
     fn subscribe(
         &mut self,
         peripheral_uuid: Uuid,
+        service_uuid: Uuid,
         characteristic_uuid: Uuid,
         fut: CoreBluetoothReplyStateShared,
     ) {
-        if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
-            if let Some(c) = p.characteristics.get_mut(&characteristic_uuid) {
-                trace!("Setting subscribe!");
-                cb::peripheral_setnotifyvalue_forcharacteristic(
-                    *p.peripheral,
-                    objc::runtime::YES,
-                    *c.characteristic,
-                );
-                c.subscribe_future_state.push_front(fut);
+        if let Some(peripheral) = self.peripherals.get_mut(&peripheral_uuid) {
+            if let Some(service) = peripheral.services.get_mut(&service_uuid) {
+                if let Some(characteristic) = service.characteristics.get_mut(&characteristic_uuid)
+                {
+                    trace!("Setting subscribe!");
+                    cb::peripheral_setnotifyvalue_forcharacteristic(
+                        *peripheral.peripheral,
+                        objc::runtime::YES,
+                        *characteristic.characteristic,
+                    );
+                    characteristic.subscribe_future_state.push_front(fut);
+                }
             }
         }
     }
@@ -550,18 +681,22 @@ impl CoreBluetoothInternal {
     fn unsubscribe(
         &mut self,
         peripheral_uuid: Uuid,
+        service_uuid: Uuid,
         characteristic_uuid: Uuid,
         fut: CoreBluetoothReplyStateShared,
     ) {
-        if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
-            if let Some(c) = p.characteristics.get_mut(&characteristic_uuid) {
-                trace!("Setting subscribe!");
-                cb::peripheral_setnotifyvalue_forcharacteristic(
-                    *p.peripheral,
-                    objc::runtime::NO,
-                    *c.characteristic,
-                );
-                c.unsubscribe_future_state.push_front(fut);
+        if let Some(peripheral) = self.peripherals.get_mut(&peripheral_uuid) {
+            if let Some(service) = peripheral.services.get_mut(&service_uuid) {
+                if let Some(characteristic) = service.characteristics.get_mut(&characteristic_uuid)
+                {
+                    trace!("Setting subscribe!");
+                    cb::peripheral_setnotifyvalue_forcharacteristic(
+                        *peripheral.peripheral,
+                        objc::runtime::NO,
+                        *characteristic.characteristic,
+                    );
+                    characteristic.unsubscribe_future_state.push_front(fut);
+                }
             }
         }
     }
@@ -579,81 +714,88 @@ impl CoreBluetoothInternal {
                     CentralDelegateEvent::DidUpdateState => {
                         self.dispatch_event(CoreBluetoothEvent::AdapterConnected).await
                     }
-                    CentralDelegateEvent::DiscoveredPeripheral(peripheral) => {
-                        self.on_discovered_peripheral(peripheral).await
+                    CentralDelegateEvent::DiscoveredPeripheral{cbperipheral} => {
+                        self.on_discovered_peripheral(cbperipheral).await
                     }
-                    CentralDelegateEvent::DiscoveredServices(peripheral_id, service_map) => {
-                        self.on_discovered_services(peripheral_id, service_map)
+                    CentralDelegateEvent::DiscoveredServices{peripheral_uuid, services} => {
+                        self.on_discovered_services(peripheral_uuid, services)
                     }
-                    CentralDelegateEvent::DiscoveredCharacteristics(peripheral_id, char_map) => {
-                        self.on_discovered_characteristics(peripheral_id, char_map)
+                    CentralDelegateEvent::DiscoveredCharacteristics{peripheral_uuid, service_uuid, characteristics} => {
+                        self.on_discovered_characteristics(peripheral_uuid, service_uuid, characteristics)
                     }
-                    CentralDelegateEvent::ConnectedDevice(peripheral_id) => {
-                        self.on_peripheral_connect(peripheral_id)
+                    CentralDelegateEvent::ConnectedDevice{peripheral_uuid} => {
+                        self.on_peripheral_connect(peripheral_uuid)
                     }
-                    CentralDelegateEvent::DisconnectedDevice(peripheral_id) => {
-                        self.on_peripheral_disconnect(peripheral_id).await
+                    CentralDelegateEvent::DisconnectedDevice{peripheral_uuid} => {
+                        self.on_peripheral_disconnect(peripheral_uuid).await
                     }
-                    CentralDelegateEvent::CharacteristicSubscribed(
-                        peripheral_id,
-                        characteristic_id,
-                    ) => self.on_characteristic_subscribed(peripheral_id, characteristic_id),
-                    CentralDelegateEvent::CharacteristicUnsubscribed(
-                        peripheral_id,
-                        characteristic_id,
-                    ) => self.on_characteristic_unsubscribed(peripheral_id, characteristic_id),
-                    CentralDelegateEvent::CharacteristicNotified(
-                        peripheral_id,
-                        characteristic_id,
+                    CentralDelegateEvent::CharacteristicSubscribed{
+                        peripheral_uuid,
+                        service_uuid,
+                        characteristic_uuid,
+                     } => self.on_characteristic_subscribed(peripheral_uuid, service_uuid, characteristic_uuid),
+                    CentralDelegateEvent::CharacteristicUnsubscribed{
+                        peripheral_uuid,
+                        service_uuid,
+                        characteristic_uuid,
+                     } => self.on_characteristic_unsubscribed(peripheral_uuid, service_uuid,characteristic_uuid),
+                    CentralDelegateEvent::CharacteristicNotified{
+                        peripheral_uuid,
+                        service_uuid,
+                        characteristic_uuid,
                         data,
-                    ) => self.on_characteristic_read(peripheral_id, characteristic_id, data).await,
-                    CentralDelegateEvent::CharacteristicWritten(
-                        peripheral_id,
-                        characteristic_id,
-                    ) => self.on_characteristic_written(peripheral_id, characteristic_id),
-                    CentralDelegateEvent::ManufacturerData(peripheral_id, manufacturer_id, manufacturer_data) => {
-                        self.on_manufacturer_data(peripheral_id, manufacturer_id, manufacturer_data).await
+                     } => self.on_characteristic_read(peripheral_uuid, service_uuid,characteristic_uuid, data).await,
+                    CentralDelegateEvent::CharacteristicWritten{
+                        peripheral_uuid,
+                        service_uuid,
+                        characteristic_uuid,
+                    } => self.on_characteristic_written(peripheral_uuid, service_uuid, characteristic_uuid),
+                    CentralDelegateEvent::ManufacturerData{peripheral_uuid, manufacturer_id, data} => {
+                        self.on_manufacturer_data(peripheral_uuid, manufacturer_id, data).await
                     },
-                    CentralDelegateEvent::ServiceData(peripheral_id, service_data) => {
-                        self.on_service_data(peripheral_id, service_data).await
+                    CentralDelegateEvent::ServiceData{peripheral_uuid, service_data} => {
+                        self.on_service_data(peripheral_uuid, service_data).await
                     },
-                    CentralDelegateEvent::Services(peripheral_id, services) => {
-                        self.on_services(peripheral_id, services).await
+                    CentralDelegateEvent::Services{peripheral_uuid, service_uuids} => {
+                        self.on_services(peripheral_uuid, service_uuids).await
                     },
                 };
             }
             adapter_msg = self.message_receiver.select_next_some() => {
                 trace!("Adapter message!");
                 match adapter_msg {
-                    CoreBluetoothMessage::StartScanning => self.start_discovery(),
+                    CoreBluetoothMessage::StartScanning{filter} => self.start_discovery(filter),
                     CoreBluetoothMessage::StopScanning => self.stop_discovery(),
-                    CoreBluetoothMessage::ConnectDevice(peripheral_uuid, fut) => {
+                    CoreBluetoothMessage::ConnectDevice{peripheral_uuid, future} => {
                         trace!("got connectdevice msg!");
-                        self.connect_peripheral(peripheral_uuid, fut);
+                        self.connect_peripheral(peripheral_uuid, future);
                     }
-                    CoreBluetoothMessage::DisconnectDevice(_peripheral_uuid, _fut) => {}
-                    CoreBluetoothMessage::ReadValue(peripheral_uuid, char_uuid, fut) => {
-                        self.read_value(peripheral_uuid, char_uuid, fut)
+                    CoreBluetoothMessage::DisconnectDevice{peripheral_uuid:_, future:_} => {}
+                    CoreBluetoothMessage::ReadValue{peripheral_uuid, service_uuid,characteristic_uuid, future} => {
+                        self.read_value(peripheral_uuid, service_uuid,characteristic_uuid, future)
                     }
-                    CoreBluetoothMessage::WriteValue(
-                        peripheral_uuid,
-                        char_uuid,
+                    CoreBluetoothMessage::WriteValue{
+                        peripheral_uuid,service_uuid,
+                        characteristic_uuid,
                         data,
-                        kind,
-                        fut,
-                    ) => self.write_value(peripheral_uuid, char_uuid, data, kind, fut),
-                    CoreBluetoothMessage::Subscribe(peripheral_uuid, char_uuid, fut) => {
-                        self.subscribe(peripheral_uuid, char_uuid, fut)
+                        write_type,
+                        future,
+                    } => self.write_value(peripheral_uuid, service_uuid,characteristic_uuid, data, write_type, future),
+                    CoreBluetoothMessage::Subscribe{peripheral_uuid, service_uuid,characteristic_uuid, future} => {
+                        self.subscribe(peripheral_uuid, service_uuid,characteristic_uuid, future)
                     }
-                    CoreBluetoothMessage::Unsubscribe(peripheral_uuid, char_uuid, fut) => {
-                        self.unsubscribe(peripheral_uuid, char_uuid, fut)
+                    CoreBluetoothMessage::Unsubscribe{peripheral_uuid, service_uuid,characteristic_uuid, future} => {
+                        self.unsubscribe(peripheral_uuid, service_uuid,characteristic_uuid, future)
+                    }
+                    CoreBluetoothMessage::IsConnected{peripheral_uuid, future} => {
+                        self.is_connected(peripheral_uuid, future);
                     }
                 };
             }
         }
     }
 
-    fn start_discovery(&mut self) {
+    fn start_discovery(&mut self, _filter: ScanFilter) {
         trace!("BluetoothAdapter::start_discovery");
         let options = ns::mutabledictionary();
         // NOTE: If duplicates are not allowed then a peripheral will not show
@@ -661,6 +803,7 @@ impl CoreBluetoothInternal {
         ns::mutabledictionary_setobject_forkey(options, ns::number_withbool(YES), unsafe {
             cb::CENTRALMANAGERSCANOPTIONALLOWDUPLICATESKEY
         });
+        // TODO: set cb::CBCENTRALMANAGERSCANOPTIONSOLICITEDSERVICEUUIDSKEY with filter.services
         cb::centralmanager_scanforperipherals_options(*self.manager, options);
     }
 
