@@ -6,109 +6,145 @@
 // for full license information.
 
 use super::{
-    adapter::uuid_to_bdaddr,
+    framework::cb::CBPeripheralState,
     internal::{
         CBPeripheralEvent, CoreBluetoothMessage, CoreBluetoothReply, CoreBluetoothReplyFuture,
     },
 };
 use crate::{
     api::{
-        self, BDAddr, CentralEvent, CharPropFlags, Characteristic, PeripheralProperties,
+        self, BDAddr, CentralEvent, CharPropFlags, Characteristic, PeripheralProperties, Service,
         ValueNotification, WriteType,
     },
-    common::{adapter_manager::AdapterManager, util},
+    common::{adapter_manager::AdapterManager, util::notifications_stream_from_broadcast_receiver},
     Error, Result,
 };
 use async_trait::async_trait;
-use futures::channel::mpsc::{self, Receiver, SendError, Sender, UnboundedSender};
+use futures::channel::mpsc::{Receiver, SendError, Sender};
 use futures::sink::SinkExt;
 use futures::stream::{Stream, StreamExt};
 use log::*;
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Serialize};
+#[cfg(feature = "serde")]
+use serde_cr as serde;
+use std::sync::Weak;
 use std::{
     collections::{BTreeSet, HashMap},
     fmt::{self, Debug, Display, Formatter},
     pin::Pin,
     sync::{Arc, Mutex},
 };
+use tokio::sync::broadcast;
 use tokio::task;
 use uuid::Uuid;
 
+#[cfg_attr(
+    feature = "serde",
+    derive(Serialize, Deserialize),
+    serde(crate = "serde_cr")
+)]
+#[derive(Clone, Debug, Eq, Hash, PartialEq)]
+pub struct PeripheralId(Uuid);
+
 /// Implementation of [api::Peripheral](crate::api::Peripheral).
 #[derive(Clone)]
 pub struct Peripheral {
-    notification_senders: Arc<Mutex<Vec<UnboundedSender<ValueNotification>>>>,
-    manager: AdapterManager<Self>,
+    shared: Arc<Shared>,
+}
+
+struct Shared {
+    notifications_channel: broadcast::Sender<ValueNotification>,
+    manager: Weak<AdapterManager<Peripheral>>,
     uuid: Uuid,
-    characteristics: Arc<Mutex<BTreeSet<Characteristic>>>,
-    properties: Arc<Mutex<PeripheralProperties>>,
+    services: Mutex<BTreeSet<Service>>,
+    properties: Mutex<PeripheralProperties>,
     message_sender: Sender<CoreBluetoothMessage>,
     // We're not actually holding a peripheral object here, that's held out in
     // the objc thread. We'll just communicate with it through our
     // receiver/sender pair.
 }
 
+impl Shared {
+    fn emit_event(&self, event: CentralEvent) {
+        if let Some(manager) = self.manager.upgrade() {
+            manager.emit(event);
+        } else {
+            trace!("Could not emit an event. AdapterManager has been dropped");
+        }
+    }
+}
+
 impl Peripheral {
     // This calls tokio::task::spawn, so it must be called from the context of a Tokio Runtime.
     pub(crate) fn new(
         uuid: Uuid,
         local_name: Option<String>,
-        manager: AdapterManager<Self>,
+        manager: Weak<AdapterManager<Self>>,
         event_receiver: Receiver<CBPeripheralEvent>,
         message_sender: Sender<CoreBluetoothMessage>,
     ) -> Self {
         // Since we're building the object, we have an active advertisement.
         // Build properties now.
-        let properties = Arc::new(Mutex::from(PeripheralProperties {
-            // Rumble required ONLY a BDAddr, not something you can get from
-            // MacOS, so we make it up for now. This sucks.
-            address: uuid_to_bdaddr(&uuid.to_string()),
+        let properties = Mutex::from(PeripheralProperties {
+            address: BDAddr::default(),
             address_type: None,
             local_name,
             tx_power_level: None,
+            rssi: None,
             manufacturer_data: HashMap::new(),
             service_data: HashMap::new(),
             services: Vec::new(),
-            discovery_count: 1,
-        }));
-        let notification_senders = Arc::new(Mutex::new(Vec::new()));
-        let ns_clone = notification_senders.clone();
-        let p_clone = properties.clone();
-        let m_clone = manager.clone();
+        });
+        let (notifications_channel, _) = broadcast::channel(16);
+
+        let shared = Arc::new(Shared {
+            properties,
+            manager,
+            services: Mutex::new(BTreeSet::new()),
+            notifications_channel,
+            uuid,
+            message_sender,
+        });
+        let shared_clone = shared.clone();
         task::spawn(async move {
             let mut event_receiver = event_receiver;
+            let shared = shared_clone;
+
             loop {
                 match event_receiver.next().await {
                     Some(CBPeripheralEvent::Notification(uuid, data)) => {
-                        util::send_notification(
-                            &ns_clone,
-                            &ValueNotification { uuid, value: data },
-                        );
+                        let notification = ValueNotification { uuid, value: data };
+
+                        // Note: we ignore send errors here which may happen while there are no
+                        // receivers...
+                        let _ = shared.notifications_channel.send(notification);
                     }
                     Some(CBPeripheralEvent::ManufacturerData(manufacturer_id, data)) => {
-                        let mut properties = p_clone.lock().unwrap();
+                        let mut properties = shared.properties.lock().unwrap();
                         properties
                             .manufacturer_data
                             .insert(manufacturer_id, data.clone());
-                        m_clone.emit(CentralEvent::ManufacturerDataAdvertisement {
-                            address: properties.address,
+                        shared.emit_event(CentralEvent::ManufacturerDataAdvertisement {
+                            id: shared.uuid.into(),
                             manufacturer_data: properties.manufacturer_data.clone(),
                         });
                     }
                     Some(CBPeripheralEvent::ServiceData(service_data)) => {
-                        let mut properties = p_clone.lock().unwrap();
+                        let mut properties = shared.properties.lock().unwrap();
                         properties.service_data.extend(service_data.clone());
 
-                        m_clone.emit(CentralEvent::ServiceDataAdvertisement {
-                            address: properties.address,
+                        shared.emit_event(CentralEvent::ServiceDataAdvertisement {
+                            id: shared.uuid.into(),
                             service_data,
                         });
                     }
                     Some(CBPeripheralEvent::Services(services)) => {
-                        let mut properties = p_clone.lock().unwrap();
+                        let mut properties = shared.properties.lock().unwrap();
                         properties.services = services.clone();
 
-                        m_clone.emit(CentralEvent::ServicesAdvertisement {
-                            address: properties.address,
+                        shared.emit_event(CentralEvent::ServicesAdvertisement {
+                            id: shared.uuid.into(),
                             services,
                         });
                     }
@@ -120,23 +156,11 @@ impl Peripheral {
                 }
             }
         });
-        Self {
-            properties,
-            manager,
-            characteristics: Arc::new(Mutex::new(BTreeSet::new())),
-            notification_senders,
-            uuid,
-            message_sender,
-        }
-    }
-
-    fn emit(&self, event: CentralEvent) {
-        debug!("emitted {:?}", event);
-        self.manager.emit(event)
+        Self { shared: shared }
     }
 
     pub(super) fn update_name(&self, name: &str) {
-        self.properties.lock().unwrap().local_name = Some(name.to_string());
+        self.shared.properties.lock().unwrap().local_name = Some(name.to_string());
     }
 }
 
@@ -153,48 +177,66 @@ impl Display for Peripheral {
 impl Debug for Peripheral {
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
         f.debug_struct("Peripheral")
-            .field("uuid", &self.uuid)
-            .field("characteristics", &self.characteristics)
-            .field("properties", &self.properties)
-            .field("message_sender", &self.message_sender)
+            .field("uuid", &self.shared.uuid)
+            .field("services", &self.shared.services)
+            .field("properties", &self.shared.properties)
+            .field("message_sender", &self.shared.message_sender)
             .finish()
     }
 }
 
 #[async_trait]
 impl api::Peripheral for Peripheral {
+    fn id(&self) -> PeripheralId {
+        PeripheralId(self.shared.uuid)
+    }
+
     fn address(&self) -> BDAddr {
-        self.properties.lock().unwrap().address
+        BDAddr::default()
     }
 
     async fn properties(&self) -> Result<Option<PeripheralProperties>> {
-        Ok(Some(self.properties.lock().unwrap().clone()))
+        Ok(Some(self.shared.properties.lock().unwrap().clone()))
     }
 
-    fn characteristics(&self) -> BTreeSet<Characteristic> {
-        self.characteristics.lock().unwrap().clone()
+    fn services(&self) -> BTreeSet<Service> {
+        self.shared.services.lock().unwrap().clone()
     }
 
     async fn is_connected(&self) -> Result<bool> {
-        // TODO
-        Ok(false)
+        let fut = CoreBluetoothReplyFuture::default();
+        self.shared
+            .message_sender
+            .to_owned()
+            .send(CoreBluetoothMessage::IsConnected {
+                peripheral_uuid: self.shared.uuid,
+                future: fut.get_state_clone(),
+            })
+            .await?;
+        match fut.await {
+            CoreBluetoothReply::State(state) => match state {
+                CBPeripheralState::Connected => Ok(true),
+                _ => Ok(false),
+            },
+            _ => panic!("Shouldn't get anything but a State!"),
+        }
     }
 
     async fn connect(&self) -> Result<()> {
         let fut = CoreBluetoothReplyFuture::default();
-        self.message_sender
+        self.shared
+            .message_sender
             .to_owned()
-            .send(CoreBluetoothMessage::ConnectDevice(
-                self.uuid,
-                fut.get_state_clone(),
-            ))
+            .send(CoreBluetoothMessage::ConnectDevice {
+                peripheral_uuid: self.shared.uuid,
+                future: fut.get_state_clone(),
+            })
             .await?;
         match fut.await {
-            CoreBluetoothReply::Connected(chars) => {
-                *(self.characteristics.lock().unwrap()) = chars;
-                self.emit(CentralEvent::DeviceConnected(
-                    self.properties.lock().unwrap().address,
-                ));
+            CoreBluetoothReply::Connected(services) => {
+                *(self.shared.services.lock().unwrap()) = services;
+                self.shared
+                    .emit_event(CentralEvent::DeviceConnected(self.shared.uuid.into()));
             }
             _ => panic!("Shouldn't get anything but connected!"),
         }
@@ -207,9 +249,9 @@ impl api::Peripheral for Peripheral {
         Ok(())
     }
 
-    async fn discover_characteristics(&self) -> Result<Vec<Characteristic>> {
-        let characteristics = self.characteristics.lock().unwrap().clone();
-        Ok(characteristics.into_iter().collect())
+    async fn discover_services(&self) -> Result<()> {
+        // TODO: Actually discover on this, rather than on connection
+        Ok(())
     }
 
     async fn write(
@@ -229,15 +271,17 @@ impl api::Peripheral for Peripheral {
         {
             write_type = WriteType::WithResponse
         }
-        self.message_sender
+        self.shared
+            .message_sender
             .to_owned()
-            .send(CoreBluetoothMessage::WriteValue(
-                self.uuid,
-                characteristic.uuid,
-                Vec::from(data),
+            .send(CoreBluetoothMessage::WriteValue {
+                peripheral_uuid: self.shared.uuid,
+                service_uuid: characteristic.service_uuid,
+                characteristic_uuid: characteristic.uuid,
+                data: Vec::from(data),
                 write_type,
-                fut.get_state_clone(),
-            ))
+                future: fut.get_state_clone(),
+            })
             .await?;
         match fut.await {
             CoreBluetoothReply::Ok => {}
@@ -248,13 +292,15 @@ impl api::Peripheral for Peripheral {
 
     async fn read(&self, characteristic: &Characteristic) -> Result<Vec<u8>> {
         let fut = CoreBluetoothReplyFuture::default();
-        self.message_sender
+        self.shared
+            .message_sender
             .to_owned()
-            .send(CoreBluetoothMessage::ReadValue(
-                self.uuid,
-                characteristic.uuid,
-                fut.get_state_clone(),
-            ))
+            .send(CoreBluetoothMessage::ReadValue {
+                peripheral_uuid: self.shared.uuid,
+                service_uuid: characteristic.service_uuid,
+                characteristic_uuid: characteristic.uuid,
+                future: fut.get_state_clone(),
+            })
             .await?;
         match fut.await {
             CoreBluetoothReply::ReadResult(chars) => Ok(chars),
@@ -266,13 +312,15 @@ impl api::Peripheral for Peripheral {
 
     async fn subscribe(&self, characteristic: &Characteristic) -> Result<()> {
         let fut = CoreBluetoothReplyFuture::default();
-        self.message_sender
+        self.shared
+            .message_sender
             .to_owned()
-            .send(CoreBluetoothMessage::Subscribe(
-                self.uuid,
-                characteristic.uuid,
-                fut.get_state_clone(),
-            ))
+            .send(CoreBluetoothMessage::Subscribe {
+                peripheral_uuid: self.shared.uuid,
+                service_uuid: characteristic.service_uuid,
+                characteristic_uuid: characteristic.uuid,
+                future: fut.get_state_clone(),
+            })
             .await?;
         match fut.await {
             CoreBluetoothReply::Ok => trace!("subscribed!"),
@@ -283,13 +331,15 @@ impl api::Peripheral for Peripheral {
 
     async fn unsubscribe(&self, characteristic: &Characteristic) -> Result<()> {
         let fut = CoreBluetoothReplyFuture::default();
-        self.message_sender
+        self.shared
+            .message_sender
             .to_owned()
-            .send(CoreBluetoothMessage::Unsubscribe(
-                self.uuid,
-                characteristic.uuid,
-                fut.get_state_clone(),
-            ))
+            .send(CoreBluetoothMessage::Unsubscribe {
+                peripheral_uuid: self.shared.uuid,
+                service_uuid: characteristic.service_uuid,
+                characteristic_uuid: characteristic.uuid,
+                future: fut.get_state_clone(),
+            })
             .await?;
         match fut.await {
             CoreBluetoothReply::Ok => {}
@@ -299,10 +349,14 @@ impl api::Peripheral for Peripheral {
     }
 
     async fn notifications(&self) -> Result<Pin<Box<dyn Stream<Item = ValueNotification> + Send>>> {
-        let (sender, receiver) = mpsc::unbounded();
-        let mut senders = self.notification_senders.lock().unwrap();
-        senders.push(sender);
-        Ok(Box::pin(receiver))
+        let receiver = self.shared.notifications_channel.subscribe();
+        Ok(notifications_stream_from_broadcast_receiver(receiver))
+    }
+}
+
+impl From<Uuid> for PeripheralId {
+    fn from(uuid: Uuid) -> Self {
+        PeripheralId(uuid)
     }
 }
 
