@@ -11,9 +11,9 @@
 //
 // Copyright (c) 2014 The Rust Project Developers
 
-use super::{ble::watcher::BLEWatcher, peripheral::Peripheral};
+use super::{ble::watcher::BLEWatcher, peripheral::Peripheral, peripheral::PeripheralId};
 use crate::{
-    api::{BDAddr, Central, CentralEvent},
+    api::{BDAddr, Central, CentralEvent, ScanFilter},
     common::adapter_manager::AdapterManager,
     Error, Result,
 };
@@ -28,13 +28,13 @@ use std::sync::{Arc, Mutex};
 #[derive(Clone)]
 pub struct Adapter {
     watcher: Arc<Mutex<BLEWatcher>>,
-    manager: AdapterManager<Peripheral>,
+    manager: Arc<AdapterManager<Peripheral>>,
 }
 
 impl Adapter {
     pub(crate) fn new() -> Self {
         let watcher = Arc::new(Mutex::new(BLEWatcher::new()));
-        let manager = AdapterManager::default();
+        let manager = Arc::new(AdapterManager::default());
         Adapter { watcher, manager }
     }
 }
@@ -55,20 +55,21 @@ impl Central for Adapter {
         Ok(self.manager.event_stream())
     }
 
-    async fn start_scan(&self) -> Result<()> {
+    async fn start_scan(&self, _filter: ScanFilter) -> Result<()> {
+        // TODO: implement filter
         let watcher = self.watcher.lock().unwrap();
         let manager = self.manager.clone();
         watcher.start(Box::new(move |args| {
             let bluetooth_address = args.BluetoothAddress().unwrap();
-            let address = bluetooth_address.try_into().unwrap();
-            if let Some(mut entry) = manager.peripheral_mut(address) {
+            let address: BDAddr = bluetooth_address.try_into().unwrap();
+            if let Some(mut entry) = manager.peripheral_mut(&address.into()) {
                 entry.value_mut().update_properties(args);
-                manager.emit(CentralEvent::DeviceUpdated(address));
+                manager.emit(CentralEvent::DeviceUpdated(address.into()));
             } else {
-                let peripheral = Peripheral::new(manager.clone(), address);
+                let peripheral = Peripheral::new(Arc::downgrade(&manager), address);
                 peripheral.update_properties(args);
-                manager.add_peripheral(address, peripheral);
-                manager.emit(CentralEvent::DeviceDiscovered(address));
+                manager.add_peripheral(peripheral);
+                manager.emit(CentralEvent::DeviceDiscovered(address.into()));
             }
         }))
     }
@@ -83,10 +84,8 @@ impl Central for Adapter {
         Ok(self.manager.peripherals())
     }
 
-    async fn peripheral(&self, address: BDAddr) -> Result<Peripheral> {
-        self.manager
-            .peripheral(address)
-            .ok_or(Error::DeviceNotFound)
+    async fn peripheral(&self, id: &PeripheralId) -> Result<Peripheral> {
+        self.manager.peripheral(id).ok_or(Error::DeviceNotFound)
     }
 
     async fn add_peripheral(&self, _address: BDAddr) -> Result<Peripheral> {
@@ -94,4 +93,9 @@ impl Central for Adapter {
             "Can't add a Peripheral from a BDAddr".to_string(),
         ))
     }
+
+    async fn adapter_info(&self) -> Result<String> {
+        // TODO: Get information about the adapter.
+        Ok("WinRT".to_string())
+    }
 }
