@@ -16,9 +16,11 @@ use crate::{api::BDAddr, winrtble::utils, Error, Result};
 use bindings::Windows::Devices::Bluetooth::GenericAttributeProfile::{
     GattCharacteristic, GattCommunicationStatus, GattDeviceService, GattDeviceServicesResult,
 };
-use bindings::Windows::Devices::Bluetooth::{BluetoothConnectionStatus, BluetoothLEDevice};
+use bindings::Windows::Devices::Bluetooth::{
+    BluetoothCacheMode, BluetoothConnectionStatus, BluetoothLEDevice,
+};
 use bindings::Windows::Foundation::{EventRegistrationToken, TypedEventHandler};
-use log::{debug, error, trace};
+use log::{debug, trace};
 
 pub type ConnectedEventHandler = Box<dyn Fn(bool) + Send>;
 
@@ -58,21 +60,37 @@ impl BLEDevice {
         })
     }
 
-    async fn get_gatt_services(&self) -> Result<GattDeviceServicesResult> {
+    async fn get_gatt_services(
+        &self,
+        cache_mode: BluetoothCacheMode,
+    ) -> Result<GattDeviceServicesResult> {
         let winrt_error = |e| Error::Other(format!("{:?}", e).into());
-        let async_op = self.device.GetGattServicesAsync().map_err(winrt_error)?;
+        let async_op = self
+            .device
+            .GetGattServicesWithCacheModeAsync(cache_mode)
+            .map_err(winrt_error)?;
         let service_result = async_op.await.map_err(winrt_error)?;
         Ok(service_result)
     }
 
     pub async fn connect(&self) -> Result<()> {
-        let service_result = self.get_gatt_services().await?;
+        if self.is_connected().await? {
+            return Ok(());
+        }
+
+        let service_result = self.get_gatt_services(BluetoothCacheMode::Uncached).await?;
         let status = service_result.Status().map_err(|_| Error::DeviceNotFound)?;
         utils::to_error(status)
     }
 
-    async fn get_characteristics(
-        &self,
+    async fn is_connected(&self) -> Result<bool> {
+        let winrt_error = |e| Error::Other(format!("{:?}", e).into());
+        let status = self.device.ConnectionStatus().map_err(winrt_error)?;
+
+        Ok(status == BluetoothConnectionStatus::Connected)
+    }
+
+    pub async fn get_characteristics(
         service: &GattDeviceService,
     ) -> std::result::Result<Vec<GattCharacteristic>, windows::Error> {
         let async_result = service.GetCharacteristicsAsync()?.await?;
@@ -87,12 +105,11 @@ impl BLEDevice {
         }
     }
 
-    pub async fn discover_characteristics(&self) -> Result<Vec<GattCharacteristic>> {
+    pub async fn discover_services(&self) -> Result<Vec<GattDeviceService>> {
         let winrt_error = |e| Error::Other(format!("{:?}", e).into());
-        let service_result = self.get_gatt_services().await?;
+        let service_result = self.get_gatt_services(BluetoothCacheMode::Cached).await?;
         let status = service_result.Status().map_err(winrt_error)?;
         if status == GattCommunicationStatus::Success {
-            let mut characteristics = Vec::new();
             // We need to convert the IVectorView to a Vec, because IVectorView is not Send and so
             // can't be help past the await point below.
             let services: Vec<_> = service_result
@@ -101,17 +118,7 @@ impl BLEDevice {
                 .into_iter()
                 .collect();
             debug!("services {:?}", services.len());
-            for service in &services {
-                match self.get_characteristics(&service).await {
-                    Ok(mut service_characteristics) => {
-                        characteristics.append(&mut service_characteristics);
-                    }
-                    Err(e) => {
-                        error!("get_characteristics_async {:?}", e);
-                    }
-                }
-            }
-            return Ok(characteristics);
+            return Ok(services);
         }
         Ok(Vec::new())
     }
